import { c as _classPrivateFieldInitSpec, d as _classPrivateFieldGet, b as _defineProperty } from '../_rollupPluginBabelHelpers-d28022b6.js';
import Service from '@ember/service';
import { assert } from '@ember/debug';
import { TrackedMap } from 'tracked-maps-and-sets';

class TargetTracker {
  constructor(name, element, options) {
    _defineProperty(this, "name", void 0);

    _defineProperty(this, "element", void 0);

    _defineProperty(this, "multiple", void 0);

    _defineProperty(this, "onChange", void 0);

    this.name = name;
    this.element = element;
    this.multiple = options.multiple ?? false;
    this.onChange = options.onChange;
  }

}

var _targets = /*#__PURE__*/new WeakMap();

var _portalCount = /*#__PURE__*/new WeakMap();

class PortalService extends Service {
  constructor(...args) {
    super(...args);

    _classPrivateFieldInitSpec(this, _targets, {
      writable: true,
      value: new TrackedMap()
    });

    _classPrivateFieldInitSpec(this, _portalCount, {
      writable: true,
      value: new TrackedMap()
    });
  }

  getTarget(name) {
    return _classPrivateFieldGet(this, _targets).get(name);
  }

  getPortalCount(name) {
    return _classPrivateFieldGet(this, _portalCount).get(name) ?? 0;
  }

  registerTarget(name, element, options) {
    assert(`Portal target with name ${name} already exists`, !_classPrivateFieldGet(this, _targets).has(name));

    _classPrivateFieldGet(this, _targets).set(name, new TargetTracker(name, element, options));
  }

  unregisterTarget(name) {
    _classPrivateFieldGet(this, _targets).delete(name);
  }

  registerPortal(name) {
    const count = (_classPrivateFieldGet(this, _portalCount).get(name) ?? 0) + 1;

    _classPrivateFieldGet(this, _portalCount).set(name, count);

    const target = this.getTarget(name);

    if (target && target.onChange) {
      target.onChange(count);
    }
  }

  unregisterPortal(name) {
    let count = _classPrivateFieldGet(this, _portalCount).get(name) ?? 0;
    assert(`Trying to unregister a portal that hasn't been registered before`, count > 0);
    count--;

    _classPrivateFieldGet(this, _portalCount).set(name, count);

    const target = this.getTarget(name);

    if (target && target.onChange) {
      target.onChange(count);
    }
  }

}

export { PortalService as default };
